/*
 *     This file is part of MediLog.
 *
 *     MediLog is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Affero General Public License as published by
 *     the Free Software Foundation.
 *
 *     MediLog is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU Affero General Public License for more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with MediLog.  If not, see <http://www.gnu.org/licenses/>.
 *
 *     Copyright (c) 2018 - 2025 by Zell-MBC.com
 */
package com.zell_mbc.medilog.base

import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.net.Uri
import android.widget.Toast
import androidx.appcompat.content.res.AppCompatResources
import androidx.compose.foundation.Image
import androidx.compose.foundation.clickable
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyListState
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.material3.Text
import androidx.compose.material3.TextFieldColors
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.State
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusManager
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.graphics.ColorFilter
import androidx.compose.ui.layout.SubcomposeLayout
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.platform.LocalSoftwareKeyboardController
import androidx.compose.ui.platform.SoftwareKeyboardController
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.AnnotatedString
import androidx.compose.ui.text.LinkAnnotation
import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.TextLinkStyles
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.withLink
import androidx.compose.ui.unit.Constraints
import androidx.compose.ui.unit.Dp
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.preference.PreferenceManager
import com.zell_mbc.medilog.MainActivity
import com.zell_mbc.medilog.MainActivity.Companion.Delimiter
import com.zell_mbc.medilog.MainActivity.Companion.LINK_COLOR
import com.zell_mbc.medilog.R
import com.zell_mbc.medilog.R.string
import com.zell_mbc.medilog.data.Data
import com.zell_mbc.medilog.data.DataViewModel
import com.zell_mbc.medilog.dialogs.toHexString
import com.zell_mbc.medilog.texttemplates.TextTemplateDialog
import com.zell_mbc.medilog.preferences.SettingsActivity
import com.zell_mbc.medilog.support.SnackbarDelegate
import com.zell_mbc.medilog.support.getCorrectedDateFormat
import com.zell_mbc.medilog.tags.TagsDialog
import java.text.DateFormat
import java.util.Calendar
import java.util.Locale
import java.util.regex.Pattern
import androidx.core.net.toUri
import com.zell_mbc.medilog.support.onAttachmentClick

abstract class BaseTab(val context: Context, val snackbarDelegate: SnackbarDelegate) {
    abstract var editActivityClass: Class<*>
    abstract var infoActivityClass: Class<*>
    abstract var chartActivityClass: Class<*>

    lateinit var viewModel: DataViewModel

    lateinit var dataList: State<List<Data>>
    lateinit var listState: LazyListState

    //lateinit var entryField1:  MutableState<String>

    // Keep values during recompositions (New)
    var value1Compose by mutableStateOf("")

    // Keep values during recompositions (Old)
    lateinit var value1:  MutableState<String>
    lateinit var value2:  MutableState<String>
    lateinit var value3:  MutableState<String>
    lateinit var value4:  MutableState<String>
    lateinit var comment: MutableState<String>
    lateinit var tags:    MutableState<String>

    lateinit var showComment: MutableState<Boolean>
    lateinit var selection: MutableState<Data?>
    lateinit var textFieldColors: TextFieldColors

    var keyboardController: SoftwareKeyboardController? = null
    var focusManager: FocusManager? = null

    val preferences: SharedPreferences = PreferenceManager.getDefaultSharedPreferences(context)
    val dateFormat: DateFormat = getCorrectedDateFormat(context)
    private val timeFormat: DateFormat = DateFormat.getTimeInstance(DateFormat.SHORT, Locale.getDefault())

    var itemUnit = "" // Get rid of
    /*
    var customTab = false
    var value1Name = "" // Hold custom name or original one
    var value1Unit = ""

    var value1Format = "" // Required to measure field width. E.g. 9.9
    var value1Length = 0 // Number of digits to be able to jump to next field
    var value1Hint = ""
*/
    var highlightValues: Boolean = false // General setting
    var highlightRow: Boolean = false    // Actual row
    var showTime: Boolean = true
    var quickEntry: Boolean = true
    var selectedField = ""

    var fontSize = 0
    var dateColumnWidth = 0.dp
    var value1Width = 0.dp
    var value2Width = 0.dp
    var value3Width = 0.dp

    val PAPERCLIP =  "\uD83D\uDCCE"

    var unit = ""
    var lowerThreshold = 0f
    var upperThreshold = 0f

    val cellPadding = 5 // Padding before and after a text in the grid
    var rowPadding = 1 // Padding tab list rows (top & bottom)
    var activateFirstField = false

    // Dialog control variables
    var showTagsDialog = mutableStateOf(false)
    var openTextTemplatesDialog = mutableStateOf(false)

    var textTemplateDialog: TextTemplateDialog
    var tagsDialog: TagsDialog

    val urlPattern: Pattern = Pattern.compile(
        "(?:^|[\\W])((ht|f)tp(s?):\\/\\/|www\\.)"
                + "(([\\w\\-]+\\.){1,}?([\\w\\-.~]+\\/?)*"
                + "[\\p{Alnum}.,%_=?&#\\-+()\\[\\]\\*$~@!:/{};']*)",
        Pattern.CASE_INSENSITIVE or Pattern.MULTILINE or Pattern.DOTALL
    )


    init {

        fontSize = context.getString(string.TEXT_SIZE_DEFAULT).toInt() // Set defined value in case the below fails
        val checkValue = preferences.getString(SettingsActivity.KEY_PREF_TEXT_SIZE, context.getString(string.TEXT_SIZE_DEFAULT))
        if (!checkValue.isNullOrEmpty()) {
            var tmp: Int
            try { tmp = checkValue.toInt()
            } catch (_: NumberFormatException) {
                tmp = context.getString(string.TEXT_SIZE_DEFAULT).toInt()
                snackbarDelegate.showSnackbar("Invalid Font Size value: $checkValue")
            }
            if (tmp > 0) fontSize = tmp
        }

        quickEntry = preferences.getBoolean(SettingsActivity.KEY_PREF_QUICKENTRY, true)
        textTemplateDialog = TextTemplateDialog(context)
        tagsDialog = TagsDialog(context)
    }

    // Helper functions to make sure UI and data are in sync
    fun updateValue1(value: String) {
        viewModel.field1.value = value
        value1Compose = value

        value1.value = value
    }
    fun updateValue2(value: String) {
        value2.value = value
        viewModel.value2 = value
    }


    @Composable
    open fun ShowContent(padding: PaddingValues) {
        // Preset fields so it survives a UI restart
        value1Compose = viewModel.field1.value

        // Initialize lateinits (old, eliminate)
        value1  = remember { mutableStateOf( viewModel.field1.value )}
        value2  = remember { mutableStateOf( viewModel.value2 )}
        value3  = remember { mutableStateOf( viewModel.value3 )}
        value4  = remember { mutableStateOf( viewModel.value4 )}
        comment = remember { mutableStateOf( viewModel.comment )}
        tags    = remember { mutableStateOf( viewModel.tags )}

        showComment = remember { mutableStateOf(false) }
       // showComment.value = comment.value.isNotEmpty()

        keyboardController = LocalSoftwareKeyboardController.current
        focusManager = LocalFocusManager.current
        textFieldColors = TextFieldDefaults.colors(unfocusedContainerColor = Color.Transparent, focusedContainerColor = Color.Transparent)

        dataList = viewModel.query().collectAsState(initial = emptyList())
        /* Todo: Check if stuff is filtered out
        if (dataList.size == 0) {
            Toast.makeText(context, context.getString(string.notEnoughDataForChart), Toast.LENGTH_LONG).show()
        }*/
        selection = remember { mutableStateOf(null) }
        listState = rememberLazyListState()

        if (dateColumnWidth == 0.dp) MeasureDateString(showTime)
    }

    abstract fun addItem()

    fun showInfoScreen(context: Context) {
        val items = viewModel.getItems("ASC", filtered = true)
        if (items.isNotEmpty()) {
            val intent = Intent(context, infoActivityClass)
            context.startActivity(intent)
        }
        else Toast.makeText(context, context.getString(string.warning) + " " + context.getString(string.noDataToShow), Toast.LENGTH_LONG).show()
    }

    fun showChartScreen(context: Context) {
        val items = viewModel.getItems("ASC", filtered = true)
        if (items.size < 2) {
            Toast.makeText(context, context.getString(string.notEnoughDataForChart), Toast.LENGTH_LONG).show()
            return
        }

        val intent = Intent(context, chartActivityClass)
        context.startActivity(intent)
    }

    // Helper functions
    @Composable
    fun MeasureColumnWidths(sampleText1: String, sampleText2: String = "", sampleText3: String = "") {
        if (value1Width == 0.dp) MeasureValue1String(sampleText1)
        if (sampleText2.isNotEmpty() && value2Width == 0.dp) MeasureValue2String(sampleText2)
        if (sampleText3.isNotEmpty() && value3Width == 0.dp) MeasureValue3String(sampleText3)
    }


    // Measure width of date string
    @Composable
    fun MeasureDateString(showTime: Boolean = true) {
        val textStyle = TextStyle(fontSize = fontSize.sp)
        val sampleCal = Calendar.getInstance()
        sampleCal.set(2015,11,17,23,5)
        val millis = sampleCal.timeInMillis
        val sampleString = if (showTime) dateFormat.format(millis) + " - " + timeFormat.format(millis) else dateFormat.format(millis)
        MeasureTextWidth(sampleString) { textWidth ->
            Text(text = "", fontSize = fontSize.sp)
            dateColumnWidth = textWidth //+ (textWidth / 10) // Add a 10% buffer
        }
    }

    // Helper functions
    @Composable
    fun MeasureValue1String(s: String) {
        MeasureTextWidth(s) { textWidth ->
            Text(text = "", fontSize = fontSize.sp)
            value1Width = textWidth
        }
    }

    @Composable
    fun MeasureValue2String(s: String) {
        MeasureTextWidth(s) { textWidth ->
            Text(text = "", fontSize = fontSize.sp)
            value2Width = textWidth
        }
    }

    @Composable
    fun MeasureValue3String(s: String) {
        MeasureTextWidth(s) { textWidth ->
            Text(text = "", fontSize = fontSize.sp)
            value3Width = textWidth
        }
    }

    @Composable
    fun MeasureTextWidth(sampleText: String, content: @Composable (width: Dp) -> Unit) {
        SubcomposeLayout { constraints ->
            val textWidth = subcompose(sampleText) {
                Text(sampleText, fontSize = fontSize.sp)
            }[0].measure(Constraints()).width.toDp()

            val contentPlaceable = subcompose("content") {
                content(textWidth)
            }[0].measure(constraints)
            layout(contentPlaceable.width, contentPlaceable.height) {
                contentPlaceable.place(0, 0)
            }
        }
    }

    @Composable
    fun ItemClicked(selectedId: Int) {
        val editItem = viewModel.getItem(selectedId)

        // Are we dealing with a genuine item or one which is blended in?
        if (editItem != null) {
            if (editItem.type != viewModel.dataType) {
                snackbarDelegate.showSnackbar(context.getString(string.blendedItem))
                return
            }
        }
        else {
            snackbarDelegate.showSnackbar("Error: Item with id $selectedId not found!")
            return
        }

        val openDialog = remember { mutableStateOf(true) }

        if (selectedField == "comment") {
            selectedField = ""
            val urlMatcher = urlPattern.matcher(editItem.comment)
            var matchStart: Int
            var matchEnd: Int
            var url = ""
            if (urlMatcher.find()) {
                matchStart = urlMatcher.start(1)
                matchEnd = urlMatcher.end()
                url = editItem.comment.substring(matchStart, matchEnd)

                val intent = Intent(Intent.ACTION_VIEW)
                intent.data = url.toUri()
                context.startActivity(intent)
            }
            else if (openDialog.value) startEditing(selectedId)
        }
        else if (openDialog.value) startEditing(selectedId)
                //showDialog.value = false

        openDialog.value = false
    }

    fun startEditing(index: Int) {
        //viewModel.editItemIndex = index
        val intent = Intent(context, editActivityClass)
        intent.putExtra("editItemIndex", index)
        context.startActivity(intent)
    }

    fun hideKeyboard() {
        /*        val v = MainActivity.mediLog.currentFocus
                if (v != null) {
                    val imm = MainActivity.mediLog.getSystemService(AppCompatActivity.INPUT_METHOD_SERVICE) as InputMethodManager
                    imm.hideSoftInputFromWindow(v.windowToken, 0)
                }*/
    }

    // Displays the icon matching the file extension
    @Composable
    fun HandleAttachment(attachment: String) {
        val colorFilter = if (isSystemInDarkTheme()) ColorFilter.tint(Color.White) else null
        Image(
            modifier = Modifier.padding(start = cellPadding.dp).clickable { onAttachmentClick(context, attachment) },
            painter = painterResource(id = getDrawable(attachment)), contentDescription = "Attachment icon",
            colorFilter = colorFilter)
    }

    @Composable
    fun HideKeyboard() {
        val controller = LocalSoftwareKeyboardController.current
        controller?.hide()
        /*val v = context.currentFocus
        if (v != null) {
            val imm = activity?.getSystemService(AppCompatActivity.INPUT_METHOD_SERVICE) as InputMethodManager
            imm.hideSoftInputFromWindow(v.windowToken, 0)
        }*/
    }

    fun showKeyboard() {
        /*val v = context.currentFocus
        if (v != null) {
            val imm = activity?.getSystemService(AppCompatActivity.INPUT_METHOD_SERVICE) as InputMethodManager
            v.requestFocus()
            imm.showSoftInput(v, 0) // Todo: Doesn't work reliably
        }*/
    }

    @Composable
    fun ShowKeyboard() {
        val controller = LocalSoftwareKeyboardController.current
        controller?.show()
    }

    fun formatDateString(m: Long): String = if (showTime) dateFormat.format(m) + " - " + timeFormat.format(m) else dateFormat.format(m)

    // Post addItem actions
    fun cleanUpAfterAddItem() {
        // New
        // Clean up UI
        value1Compose = ""

        value1.value  = ""
        value2.value  = ""
        value3.value  = ""
        value4.value  = ""
        comment.value = ""
        tags.value    = ""

        // Clean up data container
        viewModel.field1.value  = ""

        viewModel.value2  = ""
        viewModel.value3 = ""
        viewModel.value4 = ""
        viewModel.comment = ""
        viewModel.tags = ""

        hideKeyboard()
        activateFirstField = true //<- Potentially make this a setting? Annoying if you add only one item at a time
    }

    @Composable
    fun compileAnnotatedString(source: String): AnnotatedString {
        val urlMatcher = urlPattern.matcher(source)
        var matchStart: Int
        var matchEnd: Int
        var url = ""
        var retString = AnnotatedString("")
        if (urlMatcher.find()) {
            matchStart = urlMatcher.start(1)
            matchEnd = urlMatcher.end()
            url = source.substring(matchStart, matchEnd)

            retString = androidx.compose.ui.text.buildAnnotatedString {
                append(source.substring(0, matchStart))
                withLink(LinkAnnotation.Url(url = url, TextLinkStyles(style = SpanStyle(color = Color(LINK_COLOR))))) { append(url) }
                append(source.substring(matchEnd))
            }
        }
        else retString = androidx.compose.ui.text.buildAnnotatedString { append(source) }
        return retString
    }

    // Take tags, retrieve the associated color and return the color code
    fun getTagColor(tags: String): String {
        var colorString = ""

        // Todo: Pick color of first tag until I can think of a better way to show multiple tags
        val tagStrings = tags.split(Delimiter.TAG)
        if (tagStrings[0].isNotEmpty()) {
            val tmpColorString = MainActivity.tagsViewModel.getColor(tagStrings[0])
            if (tmpColorString != Color.Transparent.toHexString()) colorString = tmpColorString // Don't touch color if no tag color is set
        }
        return colorString
    }

    fun convertToString(field: Int, value: String): String {
        val inputField = when (field) {
            1 -> viewModel.field1
            2 -> viewModel.field2
            else -> viewModel.field1
        }
        var valueString = value.replace(',', '.') // Just in case

        // Making sure the value can be converted
        if (inputField.isInteger){
            // Making sure the value can be converted
            val valueNumerical = try { valueString.toInt()} catch (_: NumberFormatException) {0}
            valueString = valueNumerical.toString()
            highlightRow = (highlightValues && ((valueNumerical < lowerThreshold) || (valueNumerical > upperThreshold)))
        }
        else {
            val valueNumerical = try { valueString.toFloat() } catch (_: NumberFormatException) { 0f }
            val f = "%." + inputField.scale + "f"
            valueString = f.format(valueNumerical)
            highlightRow = (highlightValues && ((valueNumerical < lowerThreshold) || (valueNumerical > upperThreshold)))
        }
        return valueString
    }

    // Returns the correct icon based on file type
    fun getDrawable(fileName: String): Int {
        return when {
            fileName.contains(".jpg") || fileName.contains(".png") -> R.drawable.baseline_image_24
            fileName.contains(".pdf") -> R.drawable.baseline_picture_as_pdf_24
            else -> R.drawable.ic_baseline_attach_file_24
        }
    }
}
