/*
 *     This file is part of MediLog.
 *
 *     MediLog is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Affero General Public License as published by
 *     the Free Software Foundation.
 *
 *     MediLog is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU Affero General Public License for more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with MediLog.  If not, see <http://www.gnu.org/licenses/>.
 *
 *     Copyright (c) 2018 - 2025 by Zell-MBC.com
 */

/*
 */

package com.zell_mbc.medilog.glucose

import android.app.Application
import androidx.core.content.edit
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import com.zell_mbc.medilog.Tabs.GLUCOSE
import com.zell_mbc.medilog.MainActivity.Companion.Threshold
import com.zell_mbc.medilog.MainActivity.Companion.viewModel
import com.zell_mbc.medilog.R.string
import com.zell_mbc.medilog.data.DataViewModel
import com.zell_mbc.medilog.preferences.SettingsActivity
import com.zell_mbc.medilog.preferences.SettingsActivity.Companion.KEY_PREF_GLUCOSE_UNIT

class GlucoseViewModel(application: Application): DataViewModel(application, GLUCOSE) {
    override val filterStartPref = "GLUCOSEFILTERSTART"
    override val filterEndPref = "GLUCOSEFILTEREND"
    override val timeFilterModePref =  "GLUCOSE_FILTER_MODE"
    override val rollingFilterValuePref = "GLUCOSE_ROLLING_FILTER_VALUE"
    override val rollingFilterTimeframePref = "GLUCOSE_ROLLING_FILTER_TIMEFRAME"
    override val tagFilterPref = "GLUCOSE_TAG_FILTER"

    override var itemName = app.getString(string.glucose)
    override var blendInItems = preferences.getBoolean(SettingsActivity.KEY_PREF_GLUCOSE_BLENDINITEMS, app.getString(string.BLENDINITEMS_DEFAULT).toBoolean())

    var mg_dl = false

    override fun checkValues(): String {
        // Check empty variables
        if (field1.value.isEmpty()) {
            return app.getString(string.valuesMissing)
        }

        field1.value = field1.value.replace(decimalSeparator, '.')

        var pattern = ""
        if (viewModel.field1.isInteger) pattern = "\\d{1," + viewModel.field1.format.length.toString() + "}"
        else {
            val integers = field1.format.indexOf('.')
            val decimals = viewModel.field1.format.length - integers -1
            pattern = "\\d{$integers}[.,]\\d{$decimals}"
        }
        try {
            if (!Regex(pattern).matches(field1.value)) return app.getString(string.invalidFormat) + "\n" + app.getString(string.configuredFormat, "\"" + field1.format.replace('.', decimalSeparator) + "\"")
        }
        catch (_: Exception) {
            return app.getString(string.unknownError) + "\nRegex: $pattern\nValue: " + viewModel.field1.value +"\nValue: " + viewModel.field1.format
        }

        mg_dl = (field1.unit == app.getString(string.GLUCOSE_UNIT_MG_DL))

        if(mg_dl) {
            var value = validValueI(field1.value, Threshold.MAX_GLUCOSE_MG)
            if (value < 0) return app.getString(string.valuesMissing)
        }
        else {
            if (!customTab) {
                var value = validValueF(field1.value, Threshold.MAX_GLUCOSE_MMOL)
                if (value < 0) return app.getString(string.valuesMissing)
                else if (value > 10) MaterialAlertDialogBuilder(app).setTitle(app.getString(string.attention)).setMessage(app.getString(string.glucoseWrongUnit)).setPositiveButton(app.getString(string.ok)) { _, _ -> }.show()
            }
            else {
                var value = validValueF(field1.value)
                if (value < 0) return app.getString(string.valuesMissing)
            }
        }

        // Valid ketone?
        if (field2.value.isNotBlank()) {
            field2.value = field2.value.replace(decimalSeparator, '.')
            val f2 = validValueF(field2.value, Threshold.MAX_KETONE)
            if (f2 <= 0f) return app.getString(string.invalid) + " " + app.getString(string.ketone) + " " + app.getString(string.value) + " ${field2.value}"
        }

        return ""
    }

    override fun setField1Parameters() {
        val customName = "" + preferences.getString(SettingsActivity.KEY_PREF_GLUCOSE_CUSTOM_NAME, "")
        if (customName.isEmpty()) {
            customTab = false
            field1.name = app.getString(string.glucose)
            field1.unit = "" + preferences.getString(KEY_PREF_GLUCOSE_UNIT, "")
            if (field1.unit.isEmpty()) {
                field1.unit = when (app.resources.configuration.locales.get(0).toString()) {
                    "de_DE",
                    "de_CH",
                    "de_AU" -> app.getString(string.GLUCOSE_UNIT_MMOL_L)
                    else -> app.getString(string.GLUCOSE_UNIT_MG_DL)
                }
                preferences.edit { putString(KEY_PREF_GLUCOSE_UNIT, field1.unit) }
            }
            field1.format = if (field1.unit == app.getString(string.GLUCOSE_UNIT_MG_DL)) "999" else "9.9"
        } else {
            customTab = true
            field1.name = customName
            field1.unit = preferences.getString(SettingsActivity.KEY_PREF_GLUCOSE_CUSTOM_UNIT, "").toString()
            field1.format = preferences.getString(SettingsActivity.KEY_PREF_GLUCOSE_CUSTOM_DIGITS, "9.9").toString()
        }
        super.setFieldParameters(1)
    }
}
